#ifndef INPUT_KEYBOARD_H
#define INPUT_KEYBOARD_H TRUE

/*
**  keyboard.h
**
**  (C) Copyright 1996-2003 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef _WINDOWS_H

/*****************************************************************************
** Keymap file structures
*/

#define KEYMAP_NAMELEN 28

struct KeymapHeader {
   UBYTE Header[8];            /* KEYMAP10 */
   ULONG Size;                 /* sizeof(struct KeymapHeader) */
   ULONG TotalValues;          /* Total number of key values accounted for */
   ULONG MaxKeys;              /* Maximum number of keys per key value */
   UBYTE Name[KEYMAP_NAMELEN]; /* Name of the keymap */
};

/*****************************************************************************
** Methods
*/

#define MT_TranslateKeys  -1
#define MT_TranslateChars -2

struct mtTranslateKeys {
   struct dcKeyEntry *Keys;  /* List of keys to translate */
   STRING String;            /* UTF8 output buffer */
   LONG Size;                /* Size of output buffer in bytes */
};

struct mtTranslateChars {
   STRING String;            /* Character codes to translate */
   struct dcKeyEntry *Keys;  /* Output buffer */
   LONG Size;                /* Size of output buffer in bytes */
};

#endif /* _WINDOWS_H */

/****************************************************************************
** Special qualifier flags.
*/

#define KQ_LSHIFT       0x00000001  /* Left Shift is held */
#define KQ_RSHIFT       0x00000002  /* Right Shift is held*/
#define KQ_CAPSLOCK     0x00000004  /* Caps-Lock is on */
#define KQ_LCONTROL     0x00000008  /* Control Key is held */
#define KQ_RCONTROL     0x00000010  /* Control Key is held */
#define KQ_LALT         0x00000020  /* Left Alt is held */
#define KQ_RALT         0x00000040  /* Right Alt is held */
#define KQ_LCOMMAND     0x00000080  /* Left Amiga/Windows/Special */
#define KQ_RCOMMAND     0x00000100  /* Right Amiga/Windows/Special */
#define KQ_NUMPAD       0x00000200  /* This is a numeric keypad key */
#define KQ_REPEAT       0x00000400  /* This is a repeated key */
#define KQ_RELEASED     0x00000800  /* Key is now being released */
#define KQ_PRESSED      0x00001000  /* Key is being held/pressed */
#define KQ_NOTPRINTABLE 0x00002000  /* The represented key value is not printable */
#define KQ_SCROLLLOCK   0x00004000  /* Scroll Lock is on */
#define KQ_NUMLOCK      0x00008000  /* NumLock is on */
#define KQ_DEADKEY      0x00010000  /* This key will be affected by a previously pressed dead-key */

#define KQ_SHIFT    (KQ_LSHIFT|KQ_RSHIFT)
#define KQ_COMMAND  (KQ_LCOMMAND|KQ_RCOMMAND)
#define KQ_ALT      (KQ_LALT|KQ_RALT)
#define KQ_CONTROL  (KQ_LCONTROL|KQ_RCONTROL)
#define KQ_CTRL     (KQ_LCONTROL|KQ_RCONTROL)

#define KQ_INSTRUCTIONKEYS (KQ_ALT|KQ_CONTROL)
#define KQ_INFO (KQ_REPEAT|KQ_RELEASED|KQ_PRESSED|KQ_NOTPRINTABLE|KQ_CAPSLOCK)

/****************************************************************************
** Raw key presses.  Please note that when the user is using an international
** keyboard, the name of the key will not actually match what the symbol
** represents for some key types.
*/

#define K_A            1
#define K_B            2
#define K_C            3
#define K_D            4
#define K_E            5
#define K_F            6
#define K_G            7
#define K_H            8
#define K_I            9
#define K_J            10
#define K_K            11
#define K_L            12
#define K_M            13
#define K_N            14
#define K_O            15
#define K_P            16
#define K_Q            17
#define K_R            18
#define K_S            19
#define K_T            20
#define K_U            21
#define K_V            22
#define K_W            23
#define K_X            24
#define K_Y            25
#define K_Z            26
#define K_1            27
#define K_2            28
#define K_3            29
#define K_4            30
#define K_5            31
#define K_6            32
#define K_7            33
#define K_8            34
#define K_9            35
#define K_0            36
#define K_REVERSEQUOTE 37
#define K_MINUS        38
#define K_EQUALS       39
#define K_LSQUARE      40
#define K_RSQUARE      41
#define K_SEMICOLON    42
#define K_APOSTROPHE   43
#define K_COMMA        44
#define K_DOT          45
#define K_SLASH        46
#define K_BACKSLASH    47
#define K_SPACE        48

/*** Keypad keys ***/

#define K_NP0          49
#define K_NP1          50
#define K_NP2          51
#define K_NP3          52
#define K_NP4          53
#define K_NP5          54
#define K_NP6          55
#define K_NP7          56
#define K_NP8          57
#define K_NP9          58
#define K_NPMULTIPLY   59
#define K_NPPLUS       60
#define K_NPBAR        61
#define K_NPMINUS      62
#define K_NPDOT        63
#define K_NPDIVIDE     64

#undef K_CAPSLOCK
#undef K_NUMLOCK

#define K_LCONTROL     65   /* Left Control */
#define K_RCONTROL     66   /* Right Control */
#define K_HELP         67   /* Help */
#define K_LSHIFT       68   /* Left Shift  */
#define K_RSHIFT       69   /* Right Shift */
#define K_CAPSLOCK     70   /* Caps Lock   */
#define K_PRINT        71   /* Print (not print-screen!) */
#define K_LALT         72   /* Left Alt    */
#define K_RALT         73   /* Right Alt   */
#define K_LCOMMAND     74   /* Left Amiga/Win/Special  */
#define K_RCOMMAND     75   /* Right Amiga/Win/Special */
#define K_F1           76   /* Function Key 1  */
#define K_F2           77   /* Function Key 2  */
#define K_F3           78   /* Function Key 3  */
#define K_F4           79   /* Function Key 4  */
#define K_F5           80   /* Function Key 5  */
#define K_F6           81   /* Function Key 6  */
#define K_F7           82   /* Function Key 7  */
#define K_F8           83   /* Function Key 8  */
#define K_F9           84   /* Function Key 9  */
#define K_F10          85   /* Function Key 10 */
#define K_F11          86   /* Function Key 11 */
#define K_F12          87   /* Function Key 12 */
#define K_F13          88   /* Function Key 13 */
#define K_F14          89   /* Function Key 14 */
#define K_F15          90   /* Function Key 15 */
#define K_F16          91   /* Function Key 16 */
#define K_F17          92   /* Function Key 17 */
#define K_MACRO        93   /* */
#define K_NPPLUSMINUS  94   /* Plus/Minus on some numeric keypads */
#define K_LESSGREATER  95   /* Less/Greater key on foreign keyboards (no US equivalent?) */
#define K_UP           96   /* Cursor Up    */
#define K_DOWN         97   /* Cursor Down  */
#define K_RIGHT        98   /* Cursor Right */
#define K_LEFT         99   /* Cursor Left  */
#define K_SCROLL      100   /* Scroll Lock */
#define K_PAUSE       101   /* Pause */
#define K_WAKE        102   /* Wake */
#define K_SLEEP       103   /* Sleep */
#define K_POWER       104   /* Power */
#define K_BACKSPACE   105   /* Backspace */
#define K_TAB         106   /* Tabulation */
#define K_ENTER       107   /* Enter/Return */
#define K_ESCAPE      108   /* Escape */
#define K_DELETE      109   /* Delete */
#define K_CLEAR       110   /* Clear */
#define K_HOME        111   /* Home */
#define K_PAGEUP      112   /* Page Up */
#define K_PAGEDOWN    113   /* Page Down */
#define K_END         114   /* End */
#define K_SELECT      115   /* Select */
#define K_EXECUTE     116   /* Execute */
#define K_INSERT      117   /* Insert */
#define K_UNDO        118   /* Undo */
#define K_REDO        119   /* Redo */
#define K_MENU        120   /* Menu / Shortcut / Application key */
#define K_FIND        121   /* Find */
#define K_CANCEL      122   /* Cancel */
#define K_BREAK       123   /* Break */
#define K_NUMLOCK     124   /* Number lock */
#define K_PRTSCR      125   /* Print Screen */
#define K_NPENTER     126   /* Numeric keypad enter */
#define K_SYSRQ       127
#define K_F18         128
#define K_F19         129
#define K_F20         130
/*** END ***/
#define K_LIST_END    131

#endif /* INPUT_KEYBOARD_H */
